'use strict';

/**
 * @ngdoc function
 * @name alamosApp.controller:MainCtrl
 * @description
 * # MainCtrl
 * Controller of the alamosApp
 */
angular.module('alamosApp')
	.controller('FeedbackCtrl', function($scope, $location, $log, $http, $rootScope) {

 		//Init Data
		$scope.toEdit = {
			count: 0,
			groups: {}
		};

		function updateFeedback() {
			$scope.isLoading = true;
			$http.get('../feedback?password=' + $rootScope.password).
			success(function(data) {
				$log.log(data);
				$scope.data = data;
				$scope.isLoading = false;
                $scope.invalidLogin = false;
			})
			.error(function(data, status, headers, config){
				$scope.isLoading = false;
				if (status === 403){
					//Wrong password
					$scope.invalidLogin = true;
				}else{
					$scope.errorMessage = "Ein unbekannter Fehler ist aufgetreten: " + status;
				}
			});
		}

          if (angular.isDefined($location.search().password)){
              $rootScope.password = $location.search().password;
          }
  

		//Update first time
		updateFeedback();

		//Scope

		/**
			Ob ein Eintrag ein HEADER Eintrag ist oder nicht
		*/
		$scope.isHeader = function(entry){
			var isHeader = false;
			if (entry === undefined) {
				isHeader = false;
			} else {
				isHeader = entry.type === 'HEADER';
			}
			return isHeader;
		};

		/**
			Selektiert oder deselektiert einen Eintrag
		*/
		$scope.select = function(entry, group){
			$log.log(entry);
			var toEdit = $scope.toEdit.groups;

			if (toEdit[group.name] === undefined){
				//Add group
				toEdit[group.name] = [];
			}

			//Check if already exist
			var tmp = [];
			var removed = false;
			for (var i = 0; i < toEdit[group.name].length; i++) {
				var o = toEdit[group.name][i];
				if (o.name !== undefined && o.name === entry.name){
					removed = true;
					continue;
				}else{
					tmp.push(o);
				}
			}

			if (removed){
				//Removed
				$scope.toEdit.count--;
			}else{
				//Add
				$scope.toEdit.count++;
				tmp.push(entry);
			}
			toEdit[group.name] = tmp;

		};

		/**
			Verschiebt die Einheiten in die neue Gruppe
		*/
		$scope.moveEntries = function(){
			$log.log($scope.selectedGroup);

			//Create lst of all entries to move
			var toMove = [];
			for (var groupName in $scope.toEdit.groups) {
  				if ($scope.toEdit.groups.hasOwnProperty(groupName)) {
    				for (var i = 0; i < $scope.toEdit.groups[groupName].length; i++) {
    					toMove.push($scope.toEdit.groups[groupName][i]);
    				}
  				}
			}

			//Delete entries
			
			for (var i = 0; i < $scope.data.length; i++) {
				var tmp = [];
				var group = $scope.data[i];
				for (var ii = 0; ii < group.lstOfPersons.length; ii++){
					var person = group.lstOfPersons[ii];

					var shouldBeMoved = false;
					for (var iii = 0; iii < toMove.length; iii++){
						if (toMove[iii].name === person.name){
							//To move
							shouldBeMoved = true;
							break;
						}
					}


					if (!shouldBeMoved){
						//Keep
						tmp.push(person);
					}

					//Set new list of groups
					
				}
				group.lstOfPersons = tmp;
			}
			

			//Add new ones

			for (var groupName in $scope.toEdit.groups) {
				var entries = $scope.toEdit.groups[groupName];
				for (var i = 0; i < $scope.data.length; i++){
					var group = $scope.data[i];

					if (group.name === $scope.selectedGroup.name){
						//Add all to group
						group.lstOfPersons = group.lstOfPersons.concat(entries);
						break;
					}
				}
			}

			//Reset
			$scope.toEdit = {
				count: 0,
				groups: {}
			}

		};

		/**
			Wählt eine Gruppe über das Dropdown Menü aus
		*/
		$scope.selectGroup = function(group){
			$scope.selectedGroup = group;
		};

		/**
			Verschiebt eine Person innerhalb der Gruppe eine Stufe nach oben
		*/
		$scope.movePersonUp = function(person, group){
			var lstOfPersons = group.lstOfPersons;

			var indexOld = -1;
			for (var i = 0; i < lstOfPersons.length; i++){
				var p = lstOfPersons[i];
				if (p.name === person.name){
					indexOld = i;
					break;
				}
			}

			if (indexOld === -1){
				//Something went wrong
				return;
			}

			if (indexOld === 0){
				//Alread at the top
				return;
			}

			var tmp = lstOfPersons[indexOld - 1];
			lstOfPersons[indexOld] = tmp;
			lstOfPersons[indexOld-1] = person;
		};

		/**
			Verschiebt eine Person innerhalb der Gruppe eine Stufe nach unten
		*/
		$scope.movePersonDown = function(person, group){
			var lstOfPersons = group.lstOfPersons;

			var indexOld = -1;
			for (var i = 0; i < lstOfPersons.length; i++){
				var p = lstOfPersons[i];
				if (p.name === person.name){
					indexOld = i;
					break;
				}
			}

			if (indexOld === -1){
				//Something went wrong
				return;
			}

			if (indexOld === lstOfPersons.length-1){
				//Alread at the bottom
				return;
			}

			var tmp = lstOfPersons[indexOld + 1];
			lstOfPersons[indexOld] = tmp;
			lstOfPersons[indexOld+1] = person;
		};

		$scope.save = function(){
			$scope.isLoading = true;
			$http.post('../feedback?password=' + $rootScope.password, $scope.data).
			success(function(data) {
				$scope.isLoading = false;
			})
			.error(function(data, status, headers, config){
				$scope.isLoading = false;
				if (status === 403){
					//Wrong password
					$scope.invalidLogin = true;
				}else{
					$scope.errorMessage = "Ein unbekannter Fehler ist aufgetreten: " + status;
				}
			});
		};
	});